@@ -1,5 +1,6 @@
 #!python
 # coding=utf-8
+import warnings
 from collections import OrderedDict
 
 import numpy as np
@@ -43,6 +44,23 @@ class EnhancedDataset(Dataset):
         return self.get_variables_by_attributes(*args, **kwargs)
 
     def __apply_meta_interface__(self, meta):
+        warnings.warn(
+            '`__apply_meta_interface__` is deprecated. Use `apply_meta()` instead',
+            DeprecationWarning
+        )
+        return self.apply_meta(meta)
+
+    def __getattr__(self, name):
+        if name in ['__meta_interface__', '_meta']:
+            warnings.warn(
+                '`__meta_interface__` and `_meta` are deprecated. Use `meta()` instead',
+                DeprecationWarning
+            )
+            return self.meta()
+        else:
+            return super().__getattr__(name)
+
+    def apply_meta(self, meta):
         """Apply a meta interface object to a netCDF4 compatible object"""
         ds = meta.get('dimensions', OrderedDict())
         gs = meta.get('attributes', OrderedDict())
@@ -50,6 +68,9 @@ class EnhancedDataset(Dataset):
 
         # Dimensions
         for dname, dsize in ds.items():
+            # Ignore dimension sizes less than 0
+            if dsize and dsize < 0:
+                continue
             if dname not in self.dimensions:
                 self.createDimension(dname, size=dsize)
             else:
@@ -69,9 +90,12 @@ class EnhancedDataset(Dataset):
             vatts = untype_attributes(vvalue.get('attributes', {}))
 
             if vname not in self.variables:
+                if 'shape' not in vvalue and 'type' not in vvalue:
+                    L.debug("Skipping {} creation, no shape or no type defined".format(vname))
+                    continue
                 shape = vvalue.get('shape', [])  # Dimension names
                 dtype = string_to_dtype(vvalue.get('type'))
-                fillmiss = vatts.pop('_FillValue', vatts.get('missing_value', None))
+                fillmiss = vatts.get('_FillValue', vatts.get('missing_value', None))
                 newvar = self.createVariable(
                     vname,
                     dtype,
@@ -81,13 +105,18 @@ class EnhancedDataset(Dataset):
             else:
                 newvar = self.variables[vname]
 
+            # Don't re-assign fill value attributes
+            if '_FillValue' in vatts:
+                del vatts['_FillValue']
+            if 'missing_value' in vatts:
+                del vatts['missing_value']
+
             newvar.setncatts(vatts)
 
-    @property
-    def __meta_interface__(self):
+    def meta(self):
         ds = OrderedDict()
         vs = OrderedDict()
-        gs = ncpyattributes(self.__dict__)
+        gs = ncpyattributes({ ga: self.getncattr(ga) for ga in self.ncattrs() })
 
         # Dimensions
         for dname, dim in self.dimensions.items():
@@ -98,10 +127,15 @@ class EnhancedDataset(Dataset):
 
         # Variables
         for k, v in self.variables.items():
+
+            typed = v.dtype
+            if isinstance(typed, np.dtype):
+                typed = str(typed.name)
+
             vs[k] = {
-                'attributes': ncpyattributes(v.__dict__),
+                'attributes': ncpyattributes({ va: v.getncattr(va) for va in v.ncattrs() }),
                 'shape': v.dimensions,
-                'type': str(v.dtype.name)
+                'type': typed
             }
 
         return MetaInterface(
