@@ -1,16 +1,17 @@
-# -*- coding: utf-8 -*-
-from datetime import datetime
-from collections import namedtuple
-
-import netCDF4 as nc4
+#!python
+# coding=utf-8
+import six
 import numpy as np
 import pandas as pd
 
-from pygc import great_distance
-from shapely.geometry import Point, LineString
-
-from pocean.utils import unique_justseen, normalize_array, generic_masked
+from pocean.utils import (
+    normalize_array,
+    generic_masked,
+    get_masked_datetime_array
+)
 from pocean.cf import CFDataset
+from pocean.dsg.profile import profile_calculated_metadata
+
 from pocean import logger
 
 
@@ -45,7 +46,7 @@ class OrthogonalMultidimensionalProfile(CFDataset):
             assert 0 <= len(pvar.dimensions) <= 2
 
             ps = normalize_array(pvar)
-            is_single = ps.size == 1
+            is_single = isinstance(ps, six.string_types) or ps.size == 1
 
             t = dsg.t_axes()[0]
             x = dsg.x_axes()[0]
@@ -84,45 +85,7 @@ class OrthogonalMultidimensionalProfile(CFDataset):
     def calculated_metadata(self, df=None, geometries=True, clean_cols=True, clean_rows=True):
         if df is None:
             df = self.to_dataframe(clean_cols=clean_cols, clean_rows=clean_rows)
-
-        profiles = {}
-        for pid, pgroup in df.groupby('profile'):
-            pgroup = pgroup.sort_values('t')
-            first_row = pgroup.iloc[0]
-            profile = namedtuple('Profile', ['min_z', 'max_z', 't', 'x', 'y', 'loc'])
-            profiles[pid] = profile(
-                min_z=pgroup.z.min(),
-                max_z=pgroup.z.max(),
-                t=first_row.t,
-                x=first_row.x,
-                y=first_row.y,
-                loc=Point(first_row.x, first_row.y)
-            )
-
-        geometry = None
-        first_row = df.iloc[0]
-        first_loc = Point(first_row.x, first_row.y)
-        if geometries:
-            null_coordinates = df.x.isnull() | df.y.isnull()
-            coords = list(unique_justseen(zip(
-                df.x[~null_coordinates].tolist(),
-                df.y[~null_coordinates].tolist()
-            )))
-            if len(coords) > 1:
-                geometry = LineString(coords)
-            elif len(coords) == 1:
-                geometry = first_loc
-
-        meta = namedtuple('Metadata', ['min_z', 'max_z', 'min_t', 'max_t', 'profiles', 'first_loc', 'geometry'])
-        return meta(
-            min_z=df.z.min(),
-            max_z=df.z.max(),
-            min_t=df.t.min(),
-            max_t=df.t.max(),
-            profiles=profiles,
-            first_loc=first_loc,
-            geometry=geometry
-        )
+        return profile_calculated_metadata(df, geometries)
 
     def to_dataframe(self, clean_cols=True, clean_rows=True):
 
@@ -140,7 +103,7 @@ class OrthogonalMultidimensionalProfile(CFDataset):
         logger.debug(['profile data size: ', p.size])
 
         # Z
-        z = generic_masked(zvar[:], attrs=self.vatts(zvar.name)).round(5)
+        z = generic_masked(zvar[:], attrs=self.vatts(zvar.name))
         try:
             z = np.tile(z, ps)
         except ValueError:
@@ -149,42 +112,48 @@ class OrthogonalMultidimensionalProfile(CFDataset):
 
         # T
         tvar = self.t_axes()[0]
-        t = nc4.num2date(tvar[:], tvar.units, getattr(tvar, 'calendar', 'standard'))
-        if isinstance(t, datetime):
-            # Size one
-            t = np.array([t.isoformat()], dtype='datetime64')
-        t = t.repeat(zs)
-        logger.debug(['time data size: ', t.size])
+        t = tvar[:].repeat(zs)
+        nt = get_masked_datetime_array(t, tvar).flatten()
+        logger.debug(['time data size: ', nt.size])
 
         # X
         xvar = self.x_axes()[0]
-        x = generic_masked(xvar[:].repeat(zs), attrs=self.vatts(xvar.name)).round(5)
+        x = generic_masked(xvar[:].repeat(zs), attrs=self.vatts(xvar.name))
         logger.debug(['x data size: ', x.size])
 
         # Y
         yvar = self.y_axes()[0]
-        y = generic_masked(yvar[:].repeat(zs), attrs=self.vatts(yvar.name)).round(5)
+        y = generic_masked(yvar[:].repeat(zs), attrs=self.vatts(yvar.name))
         logger.debug(['y data size: ', y.size])
 
-        # Distance
-        d = np.ma.zeros(y.size, dtype=np.float64)
-        d[1:] = great_distance(start_latitude=y[0:-1], end_latitude=y[1:], start_longitude=x[0:-1], end_longitude=x[1:])['distance']
-        d = generic_masked(np.cumsum(d), minv=0).round(2)
-        logger.debug(['distance data size: ', d.size])
-
         df_data = {
-            't': t,
+            't': nt,
             'x': x,
             'y': y,
             'z': z,
-            'profile': p,
-            'distance': d
+            'profile': p
         }
 
         building_index_to_drop = np.ones(t.size, dtype=bool)
         extract_vars = list(set(self.data_vars() + self.ancillary_vars()))
         for i, dvar in enumerate(extract_vars):
-            vdata = np.ma.fix_invalid(np.ma.MaskedArray(dvar[:].round(3).flatten()))
+
+            # Profile dimension
+            if dvar.dimensions == pvar.dimensions:
+                vdata = generic_masked(dvar[:].repeat(zs).flatten(), attrs=self.vatts(dvar.name))
+
+            # Z dimension
+            elif dvar.dimensions == zvar.dimensions:
+                vdata = generic_masked(np.tile(dvar[:], ps).flatten(), attrs=self.vatts(dvar.name))
+
+            # Profile, z dimension
+            elif dvar.dimensions == pvar.dimensions + zvar.dimensions:
+                vdata = generic_masked(dvar[:].flatten(), attrs=self.vatts(dvar.name))
+
+            else:
+                logger.warning("Skipping variable {}... it didn't seem like a data variable".format(dvar))
+                continue
+
             building_index_to_drop = (building_index_to_drop == True) & (vdata.mask == True)  # noqa
             df_data[dvar.name] = vdata
 
