@@ -1,17 +1,23 @@
 #!python
 # coding=utf-8
-from collections import namedtuple, OrderedDict
+from collections import OrderedDict
 
+import six
 import numpy as np
 import pandas as pd
 import netCDF4 as nc4
-from pygc import great_distance
-from shapely.geometry import Point, LineString
 
-from pocean.utils import unique_justseen, normalize_array, get_dtype, dict_update
-from pocean.cf import CFDataset
-from pocean.cf import cf_safe_name
-from pocean import logger
+from pocean.utils import (
+    normalize_array,
+    get_dtype,
+    dict_update,
+    generic_masked,
+    get_masked_datetime_array
+)
+from pocean.cf import CFDataset, cf_safe_name
+from pocean.dsg.trajectory import trajectory_calculated_metadata
+
+from pocean import logger as L  # noqa
 
 
 class IncompleteMultidimensionalTrajectory(CFDataset):
@@ -30,6 +36,7 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
             tvars = dsg.filter_by_attrs(cf_role='trajectory_id')
             assert len(tvars) == 1
             assert dsg.featureType.lower() == 'trajectory'
+
             assert len(dsg.t_axes()) == 1
             assert len(dsg.x_axes()) == 1
             assert len(dsg.y_axes()) == 1
@@ -43,7 +50,14 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
             assert 0 <= len(tvar.dimensions) <= 2
 
             ts = normalize_array(tvar)
-            is_single = ts.size == 1
+            is_single = False
+
+            if isinstance(ts, six.string_types):
+                # Non-dimensioned string variable
+                is_single = True
+            elif ts.size == 1 and tvar.dtype != str:
+                # Other non-string types
+                is_single = True
 
             t = dsg.t_axes()[0]
             x = dsg.x_axes()[0]
@@ -55,11 +69,11 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
 
             if is_single:
                 assert len(t.dimensions) == 1
-                time_dim = dsg.dimensions[t.dimensions[0]]
+                t_dim = dsg.dimensions[t.dimensions[0]]
                 for dv in dsg.data_vars():
                     assert len(dv.dimensions) == 1
-                    assert time_dim.name in dv.dimensions
-                    assert dv.size == time_dim.size
+                    assert t_dim.name in dv.dimensions
+                    assert dv.size == t_dim.size
             else:
                 # This `time` being two dimensional is unique to IncompleteMultidimensionalTrajectory
                 assert len(t.dimensions) == 2
@@ -79,30 +93,55 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
 
     @classmethod
     def from_dataframe(cls, df, output, **kwargs):
-        reserved_columns = ['trajectory', 't', 'x', 'y', 'z', 'distance']
+        reserved_columns = ['trajectory', 't', 'x', 'y', 'z']
         data_columns = [ d for d in df.columns if d not in reserved_columns ]
 
+        axis_names = kwargs.pop('axis_names', {})
+        T_NAME = axis_names.get('t', 'time')
+        X_NAME = axis_names.get('x', 'longitude')
+        Y_NAME = axis_names.get('y', 'latitude')
+        Z_NAME = axis_names.get('z', 'z')
+
+        reduce_dims = kwargs.pop('reduce_dims', False)
+        unlimited = kwargs.pop('unlimited', False)
+
         with IncompleteMultidimensionalTrajectory(output, 'w') as nc:
 
             trajectory_group = df.groupby('trajectory')
-            max_obs = trajectory_group.size().max()
+
+            if unlimited is True:
+                max_obs = None
+            else:
+                max_obs = trajectory_group.size().max()
+            nc.createDimension('obs', max_obs)
 
             unique_trajectories = df.trajectory.unique()
-            nc.createDimension('trajectory', unique_trajectories.size)
-            nc.createDimension('obs', max_obs)
+            if reduce_dims is True and len(unique_trajectories) == 1:
+                # If a singlular trajectory, we can reduce that dimension if it is of size 1
+                def ts(t_index, size):
+                    return np.s_[0:size]
+                default_dimensions = ('obs',)
+                trajectory = nc.createVariable('trajectory', get_dtype(df.trajectory))
+            else:
+                def ts(t_index, size):
+                    return np.s_[t_index, 0:size]
+                default_dimensions = ('trajectory', 'obs')
+                nc.createDimension('trajectory', unique_trajectories.size)
+                trajectory = nc.createVariable('trajectory', get_dtype(df.trajectory), ('trajectory',))
 
             # Metadata variables
             nc.createVariable('crs', 'i4')
 
-            trajectory = nc.createVariable('trajectory', get_dtype(df.trajectory), ('trajectory',))
-
             # Create all of the variables
-            time = nc.createVariable('time', 'i4', ('trajectory', 'obs'), fill_value=int(cls.default_fill_value))
-            z = nc.createVariable('z', get_dtype(df.z), ('trajectory', 'obs'), fill_value=df.z.dtype.type(cls.default_fill_value))
-            latitude = nc.createVariable('latitude', get_dtype(df.y), ('trajectory', 'obs'), fill_value=df.y.dtype.type(cls.default_fill_value))
-            longitude = nc.createVariable('longitude', get_dtype(df.x), ('trajectory', 'obs'), fill_value=df.x.dtype.type(cls.default_fill_value))
-            if 'distance' in df:
-                distance = nc.createVariable('distance', get_dtype(df.distance), ('trajectory', 'obs'), fill_value=df.distance.dtype.type(cls.default_fill_value))
+            time = nc.createVariable(T_NAME, 'f8', default_dimensions, fill_value=np.dtype('f8').type(cls.default_fill_value))
+            time.axis = 'T'
+            time.units = cls.default_time_unit
+            z = nc.createVariable(Z_NAME, get_dtype(df.z), default_dimensions, fill_value=df.z.dtype.type(cls.default_fill_value))
+            z.axis = 'Z'
+            latitude = nc.createVariable(Y_NAME, get_dtype(df.y), default_dimensions, fill_value=df.y.dtype.type(cls.default_fill_value))
+            latitude.axis = 'Y'
+            longitude = nc.createVariable(X_NAME, get_dtype(df.x), default_dimensions, fill_value=df.x.dtype.type(cls.default_fill_value))
+            longitude.axis = 'X'
 
             attributes = dict_update(nc.nc_attributes(), kwargs.pop('attributes', {}))
 
@@ -110,17 +149,21 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
                 trajectory[i] = uid
 
                 # tolist() converts to a python datetime object without timezone
-                g = gdf.t.fillna(999999).tolist()   # 999999 is a dummy value
-                NaTs = gdf.t.isnull()
-                timenums = np.ma.MaskedArray(nc4.date2num(g, units=cls.default_time_unit))
-                timenums.mask = NaTs
-                time[i, :] = timenums
+                # and has NaTs.
+                g = gdf.t.tolist()
+                # date2num convers NaTs to np.nan
+                gg = nc4.date2num(g, units=cls.default_time_unit)
+                # masked_invalid moves np.nan to a masked value
+                time[ts(i, gg.size)] = np.ma.masked_invalid(gg)
 
-                latitude[i, :] = gdf.y.fillna(latitude._FillValue).values
-                longitude[i, :] = gdf.x.fillna(longitude._FillValue).values
-                z[i, :] = gdf.z.fillna(z._FillValue).values
-                if 'distance' in gdf:
-                    distance[i, :] = gdf.distance.fillna(distance._FillValue).values
+                lats = gdf.y.fillna(latitude._FillValue).values
+                latitude[ts(i, lats.size)] = lats
+
+                lons = gdf.x.fillna(longitude._FillValue).values
+                longitude[ts(i, lons.size)] = lons
+
+                zs = gdf.z.fillna(z._FillValue).values
+                z[ts(i, zs.size)] = zs
 
                 for c in data_columns:
                     # Create variable if it doesn't exist
@@ -128,14 +171,16 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
                     if var_name not in nc.variables:
                         if np.issubdtype(gdf[c].dtype, 'S') or gdf[c].dtype == object:
                             # AttributeError: cannot set _FillValue attribute for VLEN or compound variable
-                            v = nc.createVariable(var_name, get_dtype(gdf[c]), ('trajectory', 'obs'))
+                            v = nc.createVariable(var_name, get_dtype(gdf[c]), default_dimensions)
                         else:
-                            v = nc.createVariable(var_name, get_dtype(gdf[c]), ('trajectory', 'obs'), fill_value=gdf[c].dtype.type(cls.default_fill_value))
+                            v = nc.createVariable(var_name, get_dtype(gdf[c]), default_dimensions, fill_value=gdf[c].dtype.type(cls.default_fill_value))
 
                         if var_name not in attributes:
                             attributes[var_name] = {}
                         attributes[var_name] = dict_update(attributes[var_name], {
-                            'coordinates' : 'time latitude longitude z',
+                            'coordinates' : '{} {} {} {}'.format(
+                                T_NAME, Z_NAME, X_NAME, Y_NAME
+                            ),
                         })
                     else:
                         v = nc.variables[var_name]
@@ -146,8 +191,9 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
                         # Use an empty string... better than nothing!
                         vvalues = gdf[c].fillna('').values
 
-                    sl = slice(0, vvalues.size)
-                    v[i, sl] = vvalues
+                    # Why do we need a slice object?
+                    # sl = slice(0, vvalues.size)
+                    v[ts(i, vvalues.size)] = vvalues
 
             # Set global attributes
             nc.update_attributes(attributes)
@@ -157,75 +203,37 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
     def calculated_metadata(self, df=None, geometries=True, clean_cols=True, clean_rows=True):
         if df is None:
             df = self.to_dataframe(clean_cols=clean_cols, clean_rows=clean_rows)
-
-        trajectories = {}
-        for tid, tgroup in df.groupby('trajectory'):
-            tgroup = tgroup.sort_values('t')
-            first_row = tgroup.iloc[0]
-            first_loc = Point(first_row.x, first_row.y)
-
-            geometry = None
-            if geometries:
-                null_coordinates = tgroup.x.isnull() | tgroup.y.isnull()
-                coords = list(unique_justseen(zip(
-                    tgroup.x[~null_coordinates].tolist(),
-                    tgroup.y[~null_coordinates].tolist()
-                )))
-                if len(coords) > 1:
-                    geometry = LineString(coords)
-                elif coords == 1:
-                    geometry = first_loc
-
-            trajectory = namedtuple('Trajectory', ['min_z', 'max_z', 'min_t', 'max_t', 'first_loc', 'geometry'])
-            trajectories[tid] = trajectory(
-                min_z=tgroup.z.min(),
-                max_z=tgroup.z.max(),
-                min_t=tgroup.t.min(),
-                max_t=tgroup.t.max(),
-                first_loc=first_loc,
-                geometry=geometry
-            )
-
-        meta = namedtuple('Metadata', ['min_t', 'max_t', 'trajectories'])
-        return meta(
-            min_t=df.t.min(),
-            max_t=df.t.max(),
-            trajectories=trajectories
-        )
+        return trajectory_calculated_metadata(df, geometries)
 
     def to_dataframe(self, clean_cols=True, clean_rows=True):
         # Z
         zvar = self.z_axes()[0]
-        z = np.ma.fix_invalid(np.ma.MaskedArray(zvar[:]))
-        z = z.flatten().round(5)
-        logger.debug(['z data size: ', z.size])
+        z = generic_masked(zvar[:], attrs=self.vatts(zvar.name)).flatten()
+        L.debug(['z data size: ', z.size])
 
         # T
         tvar = self.t_axes()[0]
-        t = np.ma.MaskedArray(nc4.num2date(tvar[:], tvar.units, getattr(tvar, 'calendar', 'standard'))).flatten()
-        # Patch the time variable back to its original mask, since num2date
-        # breaks any missing/fill values
-        if hasattr(tvar[0], 'mask'):
-            t.mask = tvar[:].mask
-        logger.debug(['time data size: ', t.size])
+        t = get_masked_datetime_array(tvar[:], tvar).flatten()
+        L.debug(['time data size: ', t.size])
 
         # X
         xvar = self.x_axes()[0]
-        x = np.ma.fix_invalid(np.ma.MaskedArray(xvar[:])).flatten().round(5)
-        logger.debug(['x data size: ', x.size])
+        x = generic_masked(xvar[:], attrs=self.vatts(xvar.name)).flatten()
+        L.debug(['x data size: ', x.size])
 
         # Y
         yvar = self.y_axes()[0]
-        y = np.ma.fix_invalid(np.ma.MaskedArray(yvar[:])).flatten().round(5)
-        logger.debug(['y data size: ', y.size])
+        y = generic_masked(yvar[:], attrs=self.vatts(yvar.name)).flatten()
+        L.debug(['y data size: ', y.size])
 
         # Trajectories
         pvar = self.filter_by_attrs(cf_role='trajectory_id')[0]
-
         try:
             p = normalize_array(pvar)
+            if isinstance(p, six.string_types):
+                p = np.asarray([p])
         except BaseException:
-            logger.exception('Could not pull trajectory values from the variable, using indexes.')
+            L.exception('Could not pull trajectory values from the variable, using indexes.')
             p = np.asarray(list(range(len(pvar))), dtype=np.integer)
 
         # The Dimension that the trajectory id variable doesn't have is what
@@ -233,20 +241,14 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
         dim_diff = self.dimensions[list(set(tvar.dimensions).difference(set(pvar.dimensions)))[0]]
         if dim_diff:
             p = p.repeat(dim_diff.size)
-        logger.debug(['trajectory data size: ', p.size])
-
-        # Distance
-        d = np.append([0], great_distance(start_latitude=y[0:-1], end_latitude=y[1:], start_longitude=x[0:-1], end_longitude=x[1:])['distance'])
-        d = np.ma.fix_invalid(np.ma.MaskedArray(np.cumsum(d)).astype(np.float64).round(2))
-        logger.debug(['distance data size: ', d.size])
+        L.debug(['trajectory data size: ', p.size])
 
         df_data = OrderedDict([
             ('t', t),
             ('x', x),
             ('y', y),
             ('z', z),
-            ('trajectory', p),
-            ('distance', d),
+            ('trajectory', p)
         ])
 
         building_index_to_drop = np.ones(t.size, dtype=bool)
@@ -278,10 +280,5 @@ class IncompleteMultidimensionalTrajectory(CFDataset):
             'trajectory' : {
                 'cf_role': 'trajectory_id',
                 'long_name' : 'trajectory identifier'
-            },
-            'distance' : {
-                'long_name': 'Great circle distance between trajectory points',
-                'standard_name': 'distance_between_trajectory_points',
-                'units': 'm'
             }
         })
