@@ -1,20 +1,19 @@
 #!python
 # coding=utf-8
-from collections import namedtuple, OrderedDict
+from collections import OrderedDict
 
 import numpy as np
 import pandas as pd
-import netCDF4 as nc4
-from pygc import great_distance
-from shapely.geometry import Point, LineString
 
 from pocean.utils import (
-    unique_justseen,
+
     normalize_array,
-    get_fill_value,
-    generic_masked
+    generic_masked,
+    get_masked_datetime_array
 )
 from pocean.cf import CFDataset
+from pocean.dsg.trajectoryProfile import trajectory_profile_calculated_metadata
+
 from pocean import logger  # noqa
 
 
@@ -61,58 +60,7 @@ class ContiguousRaggedTrajectoryProfile(CFDataset):
     def calculated_metadata(self, df=None, geometries=True, clean_cols=True, clean_rows=True):
         if df is None:
             df = self.to_dataframe(clean_cols=clean_cols, clean_rows=clean_rows)
-
-        trajectories = {}
-        for tid, tgroup in df.groupby('trajectory'):
-            tgroup = tgroup.sort_values('t')
-
-            profiles = {}
-            for pid, pgroup in tgroup.groupby('profile'):
-                pgroup = pgroup.sort_values('t')
-                first_row = pgroup.iloc[0]
-                profile = namedtuple('Profile', ['min_z', 'max_z', 't', 'x', 'y', 'loc'])
-                profiles[pid] = profile(
-                    min_z=pgroup.z.min(),
-                    max_z=pgroup.z.max(),
-                    t=first_row.t,
-                    x=first_row.x,
-                    y=first_row.y,
-                    loc=Point(first_row.x, first_row.y)
-                )
-
-            geometry = None
-            first_row = tgroup.iloc[0]
-            first_loc = Point(first_row.x, first_row.y)
-            if geometries:
-                null_coordinates = tgroup.x.isnull() | tgroup.y.isnull()
-                coords = list(unique_justseen(zip(
-                    tgroup.x[~null_coordinates].tolist(),
-                    tgroup.y[~null_coordinates].tolist()
-                )))
-                if len(coords) > 1:
-                    geometry = LineString(coords)
-                elif coords == 1:
-                    geometry = first_loc
-
-            trajectory = namedtuple('Trajectory', ['min_z', 'max_z', 'min_t', 'max_t', 'profiles', 'first_loc', 'geometry'])
-            trajectories[tid] = trajectory(
-                min_z=tgroup.z.min(),
-                max_z=tgroup.z.max(),
-                min_t=tgroup.t.min(),
-                max_t=tgroup.t.max(),
-                profiles=profiles,
-                first_loc=first_loc,
-                geometry=geometry
-            )
-
-        meta = namedtuple('Metadata', ['min_z', 'max_z', 'min_t', 'max_t', 'trajectories'])
-        return meta(
-            min_z=df.z.min(),
-            max_z=df.z.max(),
-            min_t=df.t.min(),
-            max_t=df.t.max(),
-            trajectories=trajectories
-        )
+        return trajectory_profile_calculated_metadata(df, geometries)
 
     def to_dataframe(self, clean_cols=True, clean_rows=True):
         # The index variable (trajectory_index) is identified by having an
@@ -195,39 +143,23 @@ class ContiguousRaggedTrajectoryProfile(CFDataset):
             y[si:ei] = yvar[i]
             si = ei
 
-        t_mask = False
-        tfill = get_fill_value(tvar)
-        if tfill is not None:
-            t_mask = np.copy(np.ma.getmaskarray(t))
-            t[t_mask] = 1
-
-        t = np.ma.MaskedArray(
-            nc4.num2date(t, tvar.units, getattr(tvar, 'calendar', 'standard'))
-        )
-        # Patch the time variable back to its original mask, since num2date
-        # breaks any missing/fill values
-        t[t_mask] = np.ma.masked
+        #  T
+        nt = get_masked_datetime_array(t, tvar).flatten()
 
         # X and Y
-        x = generic_masked(x, minv=-180, maxv=180).round(5)
-        y = generic_masked(y, minv=-90, maxv=90).round(5)
-
-        # Distance
-        d = np.ma.zeros(o_dim.size, dtype=np.float64)
-        d[1:] = great_distance(start_latitude=y[0:-1], end_latitude=y[1:], start_longitude=x[0:-1], end_longitude=x[1:])['distance']
-        d = generic_masked(np.cumsum(d), minv=0).round(2)
+        x = generic_masked(x, minv=-180, maxv=180)
+        y = generic_masked(y, minv=-90, maxv=90)
 
         # Sample dimension
-        z = generic_masked(zvar[:].flatten(), attrs=self.vatts(zvar.name)).round(5)
+        z = generic_masked(zvar[:].flatten(), attrs=self.vatts(zvar.name))
 
         df_data = OrderedDict([
-            ('t', t),
+            ('t', nt),
             ('x', x),
             ('y', y),
             ('z', z),
             ('trajectory', r),
-            ('profile', p),
-            ('distance', d),
+            ('profile', p)
         ])
 
         building_index_to_drop = np.ones(o_dim.size, dtype=bool)
@@ -245,7 +177,7 @@ class ContiguousRaggedTrajectoryProfile(CFDataset):
 
             # Sample dimensions
             elif dvar.dimensions == (o_dim.name,):
-                vdata = generic_masked(dvar[:].flatten(), attrs=self.vatts(dvar.name)).round(3)
+                vdata = generic_masked(dvar[:].flatten(), attrs=self.vatts(dvar.name))
 
             else:
                 logger.warning("Skipping variable {}... it didn't seem like a data variable".format(dvar))
