@@ -4,6 +4,7 @@ import base64
 import operator
 import itertools
 import simplejson as json
+from datetime import datetime
 
 import numpy as np
 import netCDF4 as nc4
@@ -11,12 +12,24 @@ import netCDF4 as nc4
 from . import logger
 
 
-def all_subclasses(cls):
+def downcast_dataframe(df):
+    for column in df:
+        if np.issubdtype(df[column].dtype, np.int64):
+            df[column] = df[column].astype(np.int32)
+    return df
+
+
+def all_subclasses(cls, skips=None):
     """ Recursively generate of all the subclasses of class cls. """
+    if skips is None:
+        skips = []
+
     for subclass in cls.__subclasses__():
-        yield subclass
+        if subclass not in skips:
+            yield subclass
         for subc in all_subclasses(subclass):
-            yield subc
+            if subclass not in skips:
+                yield subc
 
 
 def unique_justseen(iterable, key=None):
@@ -102,15 +115,21 @@ def generic_masked(arr, attrs=None, minv=None, maxv=None, mask_nan=True):
         arr = np.ma.fix_invalid(arr)
 
     if isinstance(arr, np.ma.core.MaskedConstant):
-        if arr > maxv or arr < minv:
+        if arr is np.ma.masked or arr > maxv or arr < minv:
             return np.ma.masked
         return arr
+    elif arr.mask.all():
+        return arr
     else:
-        return np.ma.masked_outside(
-            arr,
+        # You can't use `masked_outside` with nan values or numpy will send a warning
+        not_nan = ~np.isnan(arr)
+        not_nan = not_nan.filled(True)
+        arr[not_nan] = np.ma.masked_outside(
+            arr[not_nan],
             minv,
             maxv
         )
+        return arr
 
 
 def pyscalar(val):
@@ -125,6 +144,25 @@ def get_fill_value(var):
     return None
 
 
+def get_masked_datetime_array(t, tvar):
+    t_mask = []
+    tfill = get_fill_value(tvar)
+    if tfill is not None:
+        t_mask = np.copy(np.ma.getmaskarray(t))
+        # Temporarily set to 1 so num2date works
+        t[t_mask] = 1
+
+    dts = nc4.num2date(t, tvar.units, getattr(tvar, 'calendar', 'standard'))
+    if isinstance(dts, datetime):
+        dts = np.array([t.isoformat()], dtype='datetime64')
+
+    # Patch the time variable back to its original mask, since num2date
+    # breaks any missing/fill values
+    nt = np.ma.MaskedArray(dts)
+    nt[t_mask] = np.ma.masked
+    return nt
+
+
 def get_dtype(obj):
     if hasattr(obj, 'dtype'):
         if obj.dtype == object:
@@ -140,11 +178,14 @@ def dict_update(d, u):
     # http://stackoverflow.com/a/3233356
     import collections
     for k, v in u.items():
-        if isinstance(v, collections.Mapping):
-            r = dict_update(d.get(k, {}), v)
-            d[k] = r
+        if isinstance(d, collections.Mapping):
+            if isinstance(v, collections.Mapping):
+                r = dict_update(d.get(k, {}), v)
+                d[k] = r
+            else:
+                d[k] = u[k]
         else:
-            d[k] = u[k]
+            d = {k: u[k] }
     return d
 
 
